#include <stdio.h>

// Behavioral view (at logic level) of a combinatorial network
// written in C and instrumented for stuck-at faults injection
// The equivalent structural view is:
//             ---
// i1|---l0---|   |
//            |AND|---l4---|
// i2|---l1---|   |        |       ----
//             ---         |------|    |
//                                |    |
//             ---         |------| OR |---l7---|o1
// i3|---l2---|   |        |      |    |
//            |AND|---l5---|  |---|    |
// i4|---l3---|   |           |    ----
//             ---            |
//                            |
// i5|---l6-------------------|
//
// i1-i5 : inputs
// l0-l7 : lines
// o1 : outputs
//
// fault and line: indicate which kind of stuck-at and on which line
// e.g. fault=1 and line=4 -> stuck-at-1 on line 4
// out-of-rang values: no faults
//
int comb_net(int i1, int i2, int i3, int i4, int i5, int fault, int line)
{
  int l0, l1, l2, l3, l4, l5, l6, l7;
  int o1;

  // Matrix for fault injection
  int sa[2][8]={{0,0,0,0,0,0,0,0},{1,1,1,1,1,1,1,1}};

  // Fault selection
  if ((fault>-1)&&(fault<2)&&(line>-1)&&(line<8))
  {
    fault=!fault;
    sa[fault][line]=!sa[fault][line];
  }
  
  // Instrumented model
  l0=(i1|sa[0][0])&sa[1][0];
  l1=(i2|sa[0][1])&sa[1][1];
  l2=(i3|sa[0][2])&sa[1][2];
  l3=(i4|sa[0][3])&sa[1][3];
  l6=(i5|sa[0][6])&sa[1][6];
  
  l4=l0&l1;
  l4=(l4|sa[0][4])&sa[1][4];

  l5=l2&l3;
  l5=(l5|sa[0][5])&sa[1][5];
  
  l7=l4|l5|l6;
  o1=(l7|sa[0][7])&sa[1][7];
  
  return o1;
}

/////////////////////////////////////////////
//Goal: to evaluate the number of test vectors for each stuck at fault

int main()
{

  // Inputs
  int i1=0,i2=0,i3=0,i4=0,i5=0;
  // Fault
  int f=-1,l=-1;
  // Counter
  int cont=0;

  // For each possible fault...
  for(f=0;f<2;f++)
    for(l=0;l<8;l++)
    {

      cont=0;

      // ...try all the inputs
      for(i1=0;i1<2;i1++)  
        for(i2=0;i2<2;i2++)  
          for(i3=0;i3<2;i3++)  
            for(i4=0;i4<2;i4++)  
              for(i5=0;i5<2;i5++)
                if (comb_net(i1,i2,i3,i4,i5,f,l)!=comb_net(i1,i2,i3,i4,i5,-1,-1)) cont++;
      printf("stuck-at-%d on line%d: %d test vectors \n", f,l,cont);
    }                       
	system("pause");
}
